#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
from decimal import Decimal

# import interfaces
from pyams_file.interfaces import IVideo, IThumbnail
from pyams_form.interfaces.form import IWidgetsPrefixViewletsManager, IFormHelp
from pyams_skin.interfaces.viewlet import IContextActions
from pyams_utils.interfaces import MANAGE_PERMISSION
from pyams_zmi.layer import IAdminLayer
from z3c.form.interfaces import HIDDEN_MODE

# import packages
from pyams_form.form import AJAXAddForm
from pyams_form.help import FormHelp
from pyams_form.schema import CloseButton
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.layer import IPyAMSLayer
from pyams_skin.viewlet.toolbar import ToolbarMenuDivider, ToolbarMenuItem
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.schema import DottedDecimalField
from pyams_viewlet.viewlet import viewlet_config, Viewlet
from pyams_zmi.form import AdminDialogAddForm
from pyramid.view import view_config
from z3c.form import field, button
from zope.interface import Interface

from pyams_media import _


@viewlet_config(name='video.thumbnail.divider', context=IVideo, layer=IPyAMSLayer, view=Interface,
                manager=IContextActions, permission=MANAGE_PERMISSION, weight=19)
class VideoDividerAction(ToolbarMenuDivider):
    """Video divider action"""


#
# Video thumbnail
#

@viewlet_config(name='video.thumbnail.action', context=IVideo, layer=IPyAMSLayer, view=Interface,
                manager=IContextActions, permission=MANAGE_PERMISSION, weight=20)
class VideoThumbnailAction(ToolbarMenuItem):
    """Video thumbnail selection action"""

    label = _("Select thumbnail...")
    label_css_class = 'fa fa-fw fa-film'

    url = 'video-thumbnail.html'
    modal_target = True


class IVideoThumbnailButtons(Interface):
    """Video thumbnail selection buttons"""

    close = CloseButton(name='close', title=_("Close"))
    select = button.Button(name='resize', title=_("Select thumbnail"))


class IVideoThumbnailInfo(Interface):
    """Video thumbnail selection info"""

    time = DottedDecimalField(title=_("Thumbnail timestamp"),
                              required=True,
                              default=Decimal(5))


@pagelet_config(name='video-thumbnail.html', context=IVideo, layer=IPyAMSLayer, permission=MANAGE_PERMISSION)
class VideoThumbnailEditForm(AdminDialogAddForm):
    """Video thumbnail selection form"""

    legend = _("Select video thumbnail")
    icon_css_class = 'fa fa-fw fa-film'

    fields = field.Fields(IVideoThumbnailInfo)
    buttons = button.Buttons(IVideoThumbnailButtons)
    ajax_handler = 'video-thumbnail.json'

    @property
    def title(self):
        return self.context.title or self.context.filename

    def updateWidgets(self, prefix=None):
        super(VideoThumbnailEditForm, self).updateWidgets(prefix)
        self.widgets['time'].mode = HIDDEN_MODE

    def updateActions(self):
        super(VideoThumbnailEditForm, self).updateActions()
        if 'select' in self.actions:
            self.actions['select'].addClass('btn-primary')

    def createAndAdd(self, data):
        thumbnailer = IThumbnail(self.context, None)
        if thumbnailer is not None:
            size = thumbnailer.get_image_size()
            time = data.get('time')
            if not isinstance(time, float):
                time = float(time)
            thumbnailer.clear_thumbnails()
            return thumbnailer.get_thumbnail('{0[0]}x{0[1]}'.format(size), 'png', time)


@view_config(name='video-thumbnail.json', context=IVideo, request_type=IPyAMSLayer,
             permission=MANAGE_PERMISSION, renderer='json', xhr=True)
class VideoThumbnailAJAXEditForm(AJAXAddForm, VideoThumbnailEditForm):
    """Video thumbnail selection form, JSON renderer"""

    def get_ajax_output(self, changes):
        translate = self.request.localizer.translate
        if changes:
            return {'status': 'success',
                    'message': translate(_("Thumbnail selected successfully."))}
        else:
            return {'status': 'info',
                    'message': translate(_("An error occurred. No created thumbnail."))}


@viewlet_config(name='video-thumbnail-prefix', context=IVideo, layer=IAdminLayer, view=VideoThumbnailEditForm,
                manager=IWidgetsPrefixViewletsManager)
@template_config(template='templates/video-thumbnail.pt')
class VideoThumbnailViewletsPrefix(Viewlet):
    """Video thumbnail edit form viewlets prefix"""


@adapter_config(context=(IVideo, IAdminLayer, VideoThumbnailEditForm), provides=IFormHelp)
class VideoThumbnailEditFormHelpAdapter(FormHelp):
    """Video thumbnail selection form help adapter"""

    message = _("""You can play the video until you display the image you want.

By pausing the video and clicking on ''Select thumbnail'' button, the selected frame will be used as
video illustration.""")
    message_format = 'rest'
